/* 	MimeDecoder.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 11-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * TODO Should use detectBoundary() with regex and use this, if no boundary was supplied		
*/
package org.brains2b.coffee.decode;

import java.io.File;

import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.local.FileMessage;

import sun.misc.BASE64Decoder;

/** Decode mime-encoded content
 *  <p>Will ignore everything that is text without a Content-Disposition
 *  and decodes other found content if it is <code>Base64</code> or
 *  <code>content-printable</code></p>
 * <p><i>Other Content-Types might be included in future releases</i></p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.00 [Aug 11, 2005]
 */
public class MimeDecoder implements Decoder {
    private final static String CONTENT_TYPE="Content-Type:";
    private final static String CONTENT_NAME="name=";
    private final static String CONTENT_ENCODING="Content-Transfer-Encoding:";
    private final static String CONTENT_DISPOSITION="Content-Disposition:";
    
    private String m_boundary;
    private FileMessage m_file=new FileMessage();
    
    public void decodePiece(String s) throws Exception {
        int idx=-1;
        int last=0;
        String boundary = m_boundary;
        if (boundary==null) {
        	boundary=CONTENT_TYPE;
        }
        idx=s.indexOf(boundary);
        while (idx<0 || (getContentValue(s,CONTENT_TYPE,idx).startsWith("text/")
        		&& !getContentValue(s,CONTENT_DISPOSITION,idx).startsWith("inline"))) {
            if (idx==-1 && last>0) {
                break;
            }
            last=idx;
            idx=s.indexOf(boundary,idx+1);
        }
         
        
        if (idx>0) {
            String fileName=getContentValue(s,CONTENT_NAME,idx);
            fileName=fileName.substring(1,fileName.length()-1);
            m_file.setFile(new File(fileName));
            String encoding = getContentValue(s,CONTENT_ENCODING,idx);
            if ("base64".equals(encoding)) { 
                BASE64Decoder dec=new BASE64Decoder();
                int start=findStart(s,idx);
                int end=findEnd(s,start);
                m_file.setData(dec.decodeBuffer(s.substring(start,end)));
            } else if ("quoted-printable".equals(encoding)) {
            	int start=findStart(s,idx);
                int end=findEnd(s,start);
                m_file.setData(s.substring(start,end).getBytes());
            }
            
        }

    }
    
    /** 
     * set the boundary as supplied by the message in
     * the header as ';boundary='
     * <p>Value does not have to be set for the Decoder
     * to work and often is not</p>
     * @param boundary String, the boundary between the 
 	 * content types
     * @see ArticleHeader#getMimeType()
     */
    public void setBoundary(String boundary) {
       m_boundary=boundary;
    }

    public FileMessage getFileMessage() throws Exception {
        return m_file;
    }
    
    /**
     * find the start of the Content for the mime-coded piece
     * @param content String
     * @param start int, the position to start looking for the content
     * @return int the start of the actual content
     */
    private int findStart(String content, int start) {
        return content.indexOf("\r\n\r\n",start)+4;
    }
  
    /**
     * find the end of the Content for the mime-coded piece
     * <p>Will look for a boundary, empty line or return
     * the length of the content if both were unsuccesfull</p>
     * @param content String
     * @param start int, the position to start looking for the end of
     * the content content
     * @return int the end of the actual content, so without the
     * boundaries or empty lines
     */
    private int findEnd(String content, int start) {
    	//TODO Should use the defined binary or detectedBoundary
    	int idx= content.indexOf("\r\n--------------",start);
    	while (idx!=-1 && !Character.isLetterOrDigit(content.charAt(idx+16))) {
    		idx=content.indexOf("\r\n--------------",idx+16);
    	}
    	if (idx!=-1) return idx;
        idx= content.indexOf("\r\n\r\n",start);
        if (idx==-1) {
        	return content.length();
        }
        return idx;
    }
    
    /**Get the value of a Mime content key from the content
     * <p>{@link #findEnd(String, int)} is used to limit the
     * content to lookin to the appropriate mime piece.
     * @param content String
     * @param key String, The content key to find
     * @param int, the position to start looking for the key
     * @return String the value belonging to the mime content key
     */
    private String getContentValue(String content, String key, int start) {
        String part = content.substring(start,findEnd(content, start));
    	int idx=part.indexOf(key,0)+key.length();
        int end=part.indexOf('\n',idx);
        if (idx>=0 && idx<end) {
            return part.substring(idx,end).trim();
        } else {
            return null;
        }
    }
}
